/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sg.materialmodule.activiti.mapper;

import com.jeeplus.core.persistence.BaseMapper;
import com.jeeplus.core.persistence.annotation.MyBatisMapper;
import com.jeeplus.modules.sg.advence.activiti.entity.Advance;
import com.jeeplus.modules.sg.managementcenter.activiti.entity.EngineeringAmount;
import com.jeeplus.modules.sg.managementcenter.constructionProject.entity.ConstructionProject;
import com.jeeplus.modules.sg.materialmodule.activiti.entity.MaterialModule;
import com.jeeplus.modules.sg.materialmodule.activiti.entity.NonPower;
import com.jeeplus.modules.sg.picking.activiti.entity.CapitalExport;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;


@MyBatisMapper
public interface MaterialModuleMapper extends BaseMapper<MaterialModule> {

    //根据项目id修改项目施工状态
    void updateConstructionStatus(@Param("status") String status,@Param("reserveProjectName") String reserveProjectName);

    //查询项目信息
    String findProjectName (String procInsId);

    //项目项目的计划开工时间
    Date findSpecificTime(String projectName);

    //查找登录名id
    List<String> findUserId(@Param("list") List<String> list);

    ConstructionProject findProject(String projectName);

    void conProject(ConstructionProject constructionProject);

    //查询当月所有的计划项目信息
    List<ConstructionProject> findMonthProject();

    //修改计划项目的是否竣工状态
    void updateEndFlag(String projectName);

    //修改配网电气含涉电报告数据-开工报告
    void updateElectricalStart(MaterialModule materialModule);

    //修改配网电气含涉电报告数据-竣工报告
    void updateElectricalEnd(MaterialModule materialModule);

    //根据储备项目名称查询项目的作业方式
    String findPlanDay(String reserveProjectName);

    List<EngineeringAmount> findProcessList(String reserveProjectName);

    void updateSeveralTimes(@Param("procInsId") String procInsId,@Param("number") int number);

    MaterialModule findById(String id);

    List<MaterialModule> findListPage(MaterialModule materialModule);
}